Window = require "clue"

function love.load()
   -- Lets create a simple new button to work with.
   local Button = Window:new("dummy", {
      x = 100, y = 100,
      width = 200, height = 100,
      bg = {normal = {30, 30, 30}, hover = {30, 30, 30}, click = {30, 30, 30}}
   })
   -- Lets also give it a few custom variables
   Button.vx = 0
   Button.vy = 0
   Button.amplifier = 100
   Button.friction = 500

   -- A update function to keep track of the mouse position, and moving according to velocity.
   function Button:update(mx, my, lmx, lmy, dt)
      -- First, lets keepe track of the last mouse position
      Button.lastmx = mx
      Button.lastmy = my

      -- If the mouse is clicked, we will move the button with mouse
      if Button:getClicked() then
         Button:setLocalX(mx - Button.startmx)
         Button:setLocalY(my - Button.startmy)
      end

      -- Move the button based on velocity
      Button:setLocalX(self:getLocalX() + (self.vx * dt))
      Button:setLocalY(self:getLocalY() + (self.vy * dt))

      -- Apply some basic friction to the velocity
      if self.vx > 0 then self.vx = math.max(0, self.vx - (self.friction * dt))
      elseif self.vx < 0 then self.vx = math.min(0, self.vx + (self.friction * dt))
      end

      if self.vy > 0 then self.vy = math.max(0, self.vy - (self.friction * dt))
      elseif self.vy < 0 then self.vy = math.min(0, self.vy + (self.friction * dt))
      end

      -- Some basic bouncing on window edges
      if self:getAbsX() < 0 then self:setAbsX(0) self.vx = self.vx * -0.5 end
      if self:getAbsX() + self:getWidth() > Window:getWidth() then self:setAbsX(Window:getWidth() - self:getWidth()) self.vx = self.vx * -0.5 end
      if self:getAbsY() < 0 then self:setAbsY(0) self.vy = self.vy * -0.5 end
      if self:getAbsY() + self:getHeight() > Window:getHeight() then self:setAbsY(Window:getHeight() - self:getHeight()) self.vy = self.vy * -0.5 end
   end

   function Button:onClick(mx, my, lmx, lmy)
      Button.startmx = lmx
      Button.startmy = lmy
   end

   -- When the button it released, we give it a velocity depending on the difference between the current and last known mouse position
   function Button:onRelease(mx, my, lmx, lmy)
      Button.vx = (mx - Button.lastmx) * self.amplifier
      Button.vy = (my - Button.lastmy) * self.amplifier
   end

   -- And last, lets give it a simple draw function
   function Button:draw()
      local x, y, w, h = self:getAbsRect()
      love.graphics.setColor(unpack(self:getCurrentBg()))

      love.graphics.rectangle("fill", x, y, w, h)
   end
end

-- This is all for updating my button library
function love.update(dt)
   local mx, my
   if OS == "Android" or OS == "iOS" then
      touches = love.touch.getTouches()
      if #touches > 0 then
         mx, my = love.touch.getPosition(touches[1])
      else
         mx, my = 0, 0
      end
   else
      mx, my = love.mouse.getPosition()
   end

   Window:update(mx, my, dt)
end

function love.draw()
   Window:draw()
end

if OS == "Android" or OS == "iOS" then
   function love.touchpressed(id, x, y) Window:onClick(x, y, 1) end
   function love.touchreleased(id, x, y) Window:onRelease(x, y, 1) end
else
   function love.mousepressed(x, y, button) Window:onClick(x, y, button) end
   function love.mousereleased(x, y, button) Window:onRelease(x, y, button) end
end
